/**
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.xfire.xmlbeans;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamReader;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.handler.AbstractHandler;
import org.codehaus.xfire.handler.Handler;

/**
 * Processes an XML document using XMLBeans
 * 
 * @version $Revision: 1.2 $
 */
public class XMLBeansHandler
    extends AbstractHandler
    implements Handler
{
    private XmlObject object;
    private XmlOptions options = new XmlOptions();
    
    public XMLBeansHandler()
    {
        options.put( XmlOptions.SAVE_NO_XML_DECL, Boolean.TRUE );
    }

    public void invoke( MessageContext context, XMLStreamReader in ) throws Exception
    {
        object = null;
        if ( in.getEventType() != XMLStreamConstants.END_ELEMENT )
        {
            object = XmlObject.Factory.parse( in );
        }
        
        handleBody( context, object );
    }
    
    /**
     * Sends the reply object to the output body
     */
    public void reply( MessageContext context, 
                       XmlObject reply ) throws Exception
    {
        // also possible to do getStreamWriter() and use that.
        reply.save( context.getResponseStream() );
    }
    
    // Properties
    //-------------------------------------------------------------------------
    /**
     * Returns the body of the SOAP request converted to the native object (POJO
     * or XmlObject or DOMish tree etc) if available or null if the body was not
     * parsed
     */
    public XmlObject getObject()
    {
        return object;
    }
    
    // Implementation methods
    //-------------------------------------------------------------------------
    protected void handleBody( MessageContext context, 
                               XmlObject body )
        throws Exception
    {
    }
}